----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:12:58 11/09/2011 
-- Design Name: 
-- Module Name:    FlashInterface - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL; -- Fr Berechnung von x <= + 1
use IEEE.numeric_std.ALL;
--********************************************************************************
entity FlashInterface is
    Port ( CLK : in    STD_LOGIC;                      -- Takt
           RST : in    STD_LOGIC;                      -- Reset
           STA : in    STD_LOGIC;                      -- Start

            WP : out   STD_LOGIC;                      -- L-aktiv, Write Protect
            RE : out   STD_LOGIC;                      -- L-aktiv, Read Enable 
            WE : out   STD_LOGIC;                      -- L-aktiv, Write Enable
            CE : out   STD_LOGIC;                      -- L-aktiv, Chip Enable

           ALE : out   STD_LOGIC;                      -- H-aktiv, Adress Latch Enable
           CLE : out   STD_LOGIC;                      -- H-aktiv, Command Latch Enable

            RB : in    STD_LOGIC;                      -- H-aktiv, Ready (=1) / Busy (=0)
           DIS : out   STD_LOGIC;                      -- H-aktiv, Disable PIC16

        IOPort : inout STD_LOGIC_VECTOR (7 downto 0));  -- Bidirectional Datenbus zum NAND-Flash

--       USBPort : out   STD_LOGIC_VECTOR (7 downto 0);  -- Sendedaten zum FTDI
--       USBWrite: out   STD_LOGIC;                      -- Write-Signal zum FTDI, Daten gltig

--        LEDs : out   STD_LOGIC_VECTOR (3 downto 0);  -- LED-Ausgnge
--         Seg : out   STD_LOGIC_VECTOR (6 downto 0);  -- 7 Segment Anzeige, Segmente A..G
--         Dig : out   STD_LOGIC_VECTOR (3 downto 0)); -- 7 Segment Anzeige, Stelle   1..4
end FlashInterface;
--********************************************************************************
architecture Behavioral of FlashInterface is
  type TReaderStatus is ( IDLE, WAIT_RDY, RESET, WAIT_RDY2, SEND_CMD1, SEND_ADR, SEND_CMD2, WAIT_RDY3, READ_DATA );
  signal ReaderStatus: TReaderStatus;

--  signal Div_CNT: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
  signal CLK20:  std_logic;                    -- 20 MHz, fr 40 MHz beide Flanken verwenden!!
--  signal DIV3:   std_logic_vector(2 downto 0); -- 60->20 MHz, NAND-Flash kann nur mit 40 MHz betrieben werden
  signal Data_In, Data_Out: std_logic_vector(7 downto 0);
  signal LE_O:    std_logic;
  signal START, ENDE:  std_logic;
--  signal Dbg_1:   STD_LOGIC_VECTOR(3 downto 0) := (others => '0');
--  signal Dbg_2:   STD_LOGIC_VECTOR(3 downto 0) := (others => '0');
--  signal Dbg_3:   STD_LOGIC_VECTOR(3 downto 0) := (others => '0');
--  signal Dbg_4:   STD_LOGIC_VECTOR(3 downto 0) := (others => '0');
---------------------------------------------------------
	component NAND_Out port(
		CLK:      IN std_logic;
		RST:      IN std_logic;
		START:    IN std_logic;
		ENDE:    OUT std_logic;
		LE:      OUT std_logic;
		WE:      OUT std_logic;
		DataIn:   IN std_logic_vector(7 downto 0);          
		DataOut: OUT std_logic_vector(7 downto 0));
	end component;
---------------------------------------------------------
--  component BinTo7Seg is
--    Port ( E1, E2, E3, E4: in STD_LOGIC_VECTOR (3 downto 0);
--           Sel:            in STD_LOGIC_VECTOR (1 downto 0);
--           Digits:        out STD_LOGIC_VECTOR (3 downto 0);
--           Segments:      out STD_LOGIC_VECTOR (6 downto 0));
--  end component;
---------------------------------------------------------
begin
  
  CLK_Proc: process(CLK, RST)
    variable DIV3: natural range 0 to 5; -- 60->20 MHz, NAND-Flash kann nur mit 40 MHz betrieben werden
  begin
    if RST = '0' then
      CLK20   <= '1';
      DIV3    := 0;
--      Div_CNT <= (others => '0');
--      Dbg_1 <= (others => '0'); Dbg_2 <= (others => '0'); Dbg_3 <= (others => '0'); Dbg_4 <= (others => '0');      
    else
      if (CLK'event) then
--        Div_CNT <= Div_CNT + 1;
        DIV3    := DIV3    + 1;
        if DIV3 = 2 then
          CLK20 <= Not CLK20;
        elsif DIV3 = 5 then
          CLK20 <= Not CLK20;
          DIV3 := 0;
        end if;
--        Dbg_1 <= std_logic_vector(to_unsigned(TReaderStatus'pos(Readerstatus), 4));
      end if;    
    end if;
  end process;
  
  Read_Proc: process(CLK20, RST)
    variable AdrCnt: natural range 0 to 4095;
  begin
    if RST = '0' then
      ReaderStatus <= IDLE;
      START <= '0';
      AdrCnt := 0;
    else
      if (CLK20'event) then
        case ReaderStatus is
          when IDLE      => if STA = '0' then 
                              ReaderStatus <= WAIT_RDY; 
                            end if;
          when WAIT_RDY  => if RB  = '1' then 
                              ReaderStatus <= RESET; 
                              START <= '1'; 
                            end if;
          when RESET     => if ENDE = '1' then
                              ReaderStatus <= WAIT_RDY2;
                              AdrCnt := 0;
                            else
                              START <= '0';
                            end if;
          when WAIT_RDY2 => if AdrCnt <= 3 then  
                              AdrCnt := AdrCnt + 1;
                            elsif RB  = '1' then 
                              ReaderStatus <= SEND_CMD1; 
                              START <= '1';
                            end if;
          when SEND_CMD1 => if ENDE = '1' then
                              ReaderStatus <= SEND_ADR;
                              START <= '1';
                              AdrCnt := 0;
                            else
                              START <= '0';
                            end if;
          when SEND_ADR  => if ENDE = '1' then
                              AdrCnt := AdrCnt + 1;
                              if AdrCnt = 4 then 
                                ReaderStatus <= SEND_CMD2; 
                              end if;
                              START <= '1';
                            else
                              START <= '0';
                            end if;
          when SEND_CMD2 => if ENDE = '1' then
                              ReaderStatus <= WAIT_RDY3;
                              AdrCnt := 0;
                            else
                              START <= '0';
                            end if;
          when WAIT_RDY3 => if AdrCnt <= 3 then  
                              AdrCnt := AdrCnt + 1;
                            elsif RB  = '1' AND CLK20 = '0' then
                              ReaderStatus <= READ_DATA;
                              AdrCnt := 0;
                            end if;
          when READ_DATA => if AdrCnt < 2112 then  
                              if CLK20 = '1' then 
                                AdrCnt := AdrCnt + 1; 
                              end if;
                            else
                              ReaderStatus <= IDLE;
                            end if;
        end case;
      end if;    
    end if;
  end process;
  
  with ReaderStatus select ALE      <= LE_O     when SEND_ADR,  '0'           when others;
  with ReaderStatus select CLE      <= LE_O     when RESET,
                                       LE_O     when SEND_CMD1,
                                       LE_O     when SEND_CMD2, '0'           when others;
  with ReaderStatus select IOPort   <= Data_Out when RESET,
                                       Data_Out when SEND_CMD1,
                                       Data_Out when SEND_ADR,
                                       Data_Out when SEND_CMD2, (others=>'Z') when others;
  with ReaderStatus select Data_In  <= x"FF"    when RESET,
                                       x"30"    when SEND_CMD2,
                                       IOPort   when READ_DATA, x"00"         when others;
--  with ReaderStatus select USBPort  <= Data_In  when READ_DATA, (others=>'Z') when others;
--  with ReaderStatus select USBWrite <= (CLK20 AND CLK AND NOT (DIV3(0) OR DIV3(1))) 
--                                                when READ_DATA,          '0'  when others;
  with ReaderStatus select RE       <= CLK20    when READ_DATA,          '1'  when others;
  
  WP  <= '1'; -- niemals
  CE  <= '0'; -- immer
  DIS <= '1'; -- immer

--  LEDs(0) <= '1';
--  LEDs(1) <= '0';
--  LEDs(2) <= '1';
--  LEDs(3) <= '0';

	I_Out:  NAND_Out  PORT MAP (CLK20, RST, START, ENDE, LE_O, WE, Data_In, Data_Out );
--  I_7Seg: BinTo7Seg PORT MAP (Dbg_1, Dbg_2, Dbg_3, Dbg_4, Div_CNT(15 downto 14), Dig, Seg);
end Behavioral;
--********************************************************************************
